module Aspect
  # Extend in a `Class` to register new instances with a name in a collection named `.registry`.
  module HasRegistry # TODO: Allow to register Classes, not just instances
    # All defined instances.
    #
    # @return [{Symbol => Object}]
    def registry
      @registry ||= {}
    end

    # Register an instance.
    #
    # @param [#to_sym] name
    # @yield The new instance's scope.
    # @return [Object]
    def register(name, *arguments, &block)
      registry[name.to_sym] = new(*arguments, &block)
    end

    # Get whether an instance is registered.
    #
    # @param [#to_sym] name
    # @return [Boolean]
    def registered?(name)
      @registry.keys.include?(name.to_sym)
    end
  end
end
